/************************************************************************/
/* @\F pPbg_TOPW[                                 */
/*                                                                      */
/* TvF AXI Litef[^oX:32bitpPbg֕ϊ܂                */
/*                                                                      */
/*                                                                  */
/*                                                                      */
/*                                                                      */
/* ύXF                                                           */
/*  +------------------ ԍ (000 ` 999)                           */
/*  |    +------------- VKAύXAǉA폜̕                    */
/*  v    v                                                              */
/*  No     N                                               */
/* ----+------+----------+--------------------------------------------- */
/*  000 VK   13/01/15                                                 */
/*                                                                      */
/************************************************************************/

`timescale 1ns/1ps

module smt_packet_axi_lite_slave #
	(
	parameter integer C_S_AXI_ADDR_WIDTH	= 32,
	parameter integer C_S_AXI_DATA_WIDTH	= 32
	)
	(
	// System Signals
	input wire			ACLK,
	input wire			ARESETN,

	// Slave Interface Write Address Ports
	input  wire	[C_S_AXI_ADDR_WIDTH-1:0] S_AXI_AWADDR,
	input  wire	[3-1:0]		S_AXI_AWPROT,
	input  wire			S_AXI_AWVALID,
	output wire			S_AXI_AWREADY,

	// Slave Interface Write Data Ports
	input  wire	[C_S_AXI_DATA_WIDTH-1:0] S_AXI_WDATA,
	input  wire	[C_S_AXI_DATA_WIDTH/8-1:0] S_AXI_WSTRB,
	input  wire			S_AXI_WVALID,
	output wire			S_AXI_WREADY,

	// Slave Interface Write Response Ports
	output wire	[2-1:0]		S_AXI_BRESP,
	output wire			S_AXI_BVALID,
	input  wire			S_AXI_BREADY,

	// Slave Interface Read Address Ports
	input  wire	[C_S_AXI_ADDR_WIDTH-1:0] S_AXI_ARADDR,
	input  wire	[3-1:0]		S_AXI_ARPROT,
	input  wire			S_AXI_ARVALID,
	output wire			S_AXI_ARREADY,

	// Slave Interface Read Data Ports
	output wire	[C_S_AXI_DATA_WIDTH-1:0] S_AXI_RDATA,
	output wire	[2-1:0]		S_AXI_RRESP,
	output wire			S_AXI_RVALID,
	input  wire			S_AXI_RREADY,

	// Packet I/F for System Macro Trace
	input  wire			smt_sync_clk,	// up to 100MHz
	output wire			clk_smt,
	output wire			xsync_smt,
	output wire	[3:0]		d_smt
	);


/***************************************************************************/
/* Parameter                                                               */
/***************************************************************************/
 parameter	PACKET_IF_ADR =	20'h40000;	//Packet I/F Address = 0x400000000 to 0x40000FFF

 parameter	RESP_OKAY =	2'b00;
 parameter	RESP_EXOKAY =	2'b01;
 parameter	RESP_SLVERR =	2'b10;
 parameter	RESP_DECERR =	2'b11;

 parameter	IDLE_WR    =	2'b00,
		WAIT_WR_DATA =	2'b01,
		WR_END =	2'b10;

 parameter	IDLE_RD    =	2'b00,
		RD_DATA_WAIT =	2'b01,
		RD_DATA_VALID =	2'b10;

/***************************************************************************/
/* node signals                                                            */
/***************************************************************************/
 reg	[1:0]	wr_acs_st;
 reg		xtcs;
 reg	[4:2]	ta;
 reg	[31:0]	td;
 reg		xtwr;

 reg		rvalid;
 reg	[1:0]	rd_acs_st;
 reg	[31:0]	dummy_data;

 reg		axi_bvalid;
 reg		axi_wready;


/***************************************************************************/
/* State Machine for Write Access                                          */
/***************************************************************************/
	always @(posedge ACLK or negedge ARESETN) begin
		if (~ARESETN) begin
			xtcs <= 1'b1;
			ta[4:2] <= 0;
			td[31:0] <= 0;
			xtwr <= 1'b1;
			wr_acs_st <= IDLE_WR;
			axi_wready <= 1'b0;
			axi_bvalid <= 1'b0;
		end
		else begin
			case (wr_acs_st)
			IDLE_WR : begin
				axi_bvalid <= 1'b0;
				if (S_AXI_AWVALID) begin
					xtcs <= ~(S_AXI_AWADDR[31:12] == PACKET_IF_ADR);
					ta[4:2] <= S_AXI_AWADDR[4:2];
					if (S_AXI_WVALID) begin
						td[31:0] <= S_AXI_WDATA[31:0] ;
						xtwr <= 1'b0;
						axi_wready <= 1'b1;
						wr_acs_st <= WR_END;
					end
					else begin
						td[31:0] <= td[31:0] ;
						xtwr <= 1'b1;
						axi_wready <= 1'b0;
						wr_acs_st <= WAIT_WR_DATA;
					end
				end
				else begin
					axi_wready <= 1'b0;
					xtcs <= 1'b1;
					ta[4:2] <= ta[4:2];
					td[31:0] <= td[31:0];
					xtwr <= 1'b1;
					wr_acs_st <= IDLE_WR;
				end
			end
			WAIT_WR_DATA: begin
				axi_bvalid <= 1'b0;
				xtcs <= 1'b0;
				ta[4:2] <= ta[4:2];
				if (S_AXI_WVALID) begin
					td[31:0] <= S_AXI_WDATA[31:0];
					xtwr <= 1'b0;
					axi_wready <= 1'b1;
					wr_acs_st <= WR_END;
				end
				else begin
					td[31:0] <= td[31:0] ;
					xtwr <= 1'b1;
					axi_wready <= 1'b0;
					wr_acs_st <= WAIT_WR_DATA;
				end
			end
			WR_END: begin
				axi_bvalid <= 1'b1;
				axi_wready <= 1'b0;
				xtcs <= 1'b0;
				ta[4:2] <= ta[4:2];
				td[31:0] <= td[31:0];
				xtwr <= 1'b1;
				wr_acs_st <= IDLE_WR;
			end
			default: begin
				axi_bvalid <= 1'b0;
				axi_wready <= 1'b0;
				xtcs <= 1'b1;
				ta[4:2] <= ta[4:2];
				td[31:0] <= td[31:0];
				xtwr <= 1'b1;
				wr_acs_st <= IDLE_WR;
			end
			endcase
		end
	end


/***************************************************************************/
/* Write Response                                                          */
/***************************************************************************/
	assign S_AXI_WREADY = axi_wready;
	assign S_AXI_BVALID = axi_bvalid;
	assign S_AXI_AWREADY = 1'b1;
	assign S_AXI_BRESP = RESP_OKAY;


/***************************************************************************/
/* State Machine for Read Access (Dummy)                                   */
/***************************************************************************/
	always @(posedge ACLK or negedge ARESETN) begin
		if (~ARESETN) begin
			rvalid <= 1'b0;
			rd_acs_st <= IDLE_RD;
		end
		else begin
			case (rd_acs_st)
			IDLE_RD : begin
				if (S_AXI_ARVALID) begin
					rd_acs_st <= RD_DATA_WAIT;
				end
				else begin
					rd_acs_st <= IDLE_RD;
				end
			end
			RD_DATA_WAIT : begin
				rvalid <= 1'b1;
				rd_acs_st <= RD_DATA_VALID;
			end
			RD_DATA_VALID : begin
				if (S_AXI_RREADY) begin
					rvalid <= 1'b0;
					rd_acs_st <= IDLE_RD;
				end
				else begin
					rvalid <= 1'b1;
					rd_acs_st <= RD_DATA_VALID;
				end
			end
			endcase
		end
	end

	assign S_AXI_RVALID = rvalid;

/***************************************************************************/
/* Read Response (Dummy)                                                   */
/***************************************************************************/
	always @(posedge ACLK or negedge ARESETN) begin
		if (~ARESETN)
			dummy_data <= 32'hffff_ffff;
		else begin
			if (rd_acs_st == RD_DATA_WAIT) begin
				dummy_data <= 32'h1234_5678;
			end
			else begin
				dummy_data <= 32'h0000_0000;
			end
		end
	end

	assign S_AXI_RDATA = dummy_data;
	assign S_AXI_ARREADY = 1'b1;
	assign S_AXI_RRESP = RESP_OKAY;


/***************************************************************************/
/* Packet I/F for System Macro Trace                                       */
/***************************************************************************/

port_top port_top_inst(
	.busclk(ACLK),
	.trclk(smt_sync_clk),
	.reset(~ARESETN),
	.xtcs(xtcs),
	.ta(ta),
	.td(td),
	.xtwr(xtwr),
	.clk(clk_smt),
	.xsync(xsync_smt),
	.d(d_smt)
	);

endmodule
